@@ -14,18 +14,20 @@ SPDX-License-Identifier: MIT
 namespace IGC::detail {
 class SignalGuard {
 public:
-  SignalGuard(int Signal, void (*Handler)(int, siginfo_t *, void *))
-      : Signal(Signal) {
+  SignalGuard(int Signal, void (*Handler)(int, siginfo_t *, void *)) : Signal(Signal) {
     sigaction(Signal, nullptr, &SAOld);
-    struct sigaction SA{};
-    sigemptyset(&SA.sa_mask);
-    SA.sa_sigaction = Handler;
-    SA.sa_flags = 0;
-    sigaction(Signal, &SA, nullptr);
+    if (SAOld.sa_handler == SIG_DFL) {
+      struct sigaction SA;
+      sigemptyset(&SA.sa_mask);
+      SA.sa_sigaction = Handler;
+      SA.sa_flags = 0;
+      sigaction(Signal, &SA, nullptr);
+    }
   }
 
   ~SignalGuard() {
-    sigaction(Signal, &SAOld, nullptr);
+    if (SAOld.sa_handler == SIG_DFL)
+      sigaction(Signal, &SAOld, nullptr);
   }
 
   SignalGuard(const SignalGuard &) = delete;
@@ -37,8 +39,7 @@ private:
 }; // class SignalGuard
 } // namespace IGC::detail
 
-#define SET_SIG_HANDLER(SIG)                                                   \
-  ::IGC::detail::SignalGuard SG##SIG(SIG, signalHandler);
+#define SET_SIG_HANDLER(SIG) ::IGC::detail::SignalGuard SG##SIG(SIG, signalHandler);
 
 #define REMOVE_SIG_HANDLER(SIG) (void)SG##SIG;
 
